import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * {@code Grid} instances represent the grid in <i>The Game of Life</i>.
 */
public class Grid implements Iterable<Cell> {

    private final int numberOfRows;
    private final int numberOfColumns;
    private final Cell[][] cells;

    /**
     * Creates a new {@code Grid} instance given the number of rows and columns.
     *
     * @param numberOfRows    the number of rows
     * @param numberOfColumns the number of columns
     * @throws IllegalArgumentException if {@code numberOfRows} or {@code numberOfColumns} are
     *                                  less than or equal to 0
     */
    public Grid(int numberOfRows, int numberOfColumns) {
        this.numberOfRows = numberOfRows;
        this.numberOfColumns = numberOfColumns;
        this.cells = createCells();
    }

    @Override
    public Iterator<Cell> iterator() {
        return new GridIterator(this);
    }

    private Cell[][] createCells() {
        Cell[][] cells = new Cell[getNumberOfRows()][getNumberOfColumns()];
        for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
                cells[rowIndex][columnIndex] = new Cell();
            }
        }
        return cells;
    }

    /**
     * Returns the {@link Cell} at the given index.
     *
     * <p>Note that the index is wrapped around so that a {@link Cell} is always returned.
     *
     * @param rowIndex    the row index of the {@link Cell}
     * @param columnIndex the column index of the {@link Cell}
     * @return the {@link Cell} at the given row and column index
     */
    public Cell getCell(int rowIndex, int columnIndex) {
        return cells[getWrappedRowIndex(rowIndex)][getWrappedColumnIndex(columnIndex)];
    }

    private int getWrappedRowIndex(int rowIndex) {
        return (rowIndex + getNumberOfRows()) % getNumberOfRows();
    }

    private int getWrappedColumnIndex(int columnIndex) {
        return (columnIndex + getNumberOfColumns()) % getNumberOfColumns();
    }

    /**
     * Returns the number of rows in this {@code Grid}.
     *
     * @return the number of rows in this {@code Grid}
     */
    public int getNumberOfRows() {
        return numberOfRows;
    }

    /**
     * Returns the number of columns in this {@code Grid}.
     *
     * @return the number of columns in this {@code Grid}
     */
    public int getNumberOfColumns() {
        return numberOfColumns;
    }

    /**
     * Transitions all {@link Cell}s in this {@code Grid} to the next generation.
     *
     * <p>The following rules are applied:
     * <ul>
     * <li>Any live {@link Cell} with fewer than two live neighbours dies, i.e. underpopulation.</li>
     * <li>Any live {@link Cell} with two or three live neighbours lives on to the next
     * generation.</li>
     * <li>Any live {@link Cell} with more than three live neighbours dies, i.e. overpopulation.</li>
     * <li>Any dead {@link Cell} with exactly three live neighbours becomes a live cell, i.e.
     * reproduction.</li>
     * </ul>
     */
    void nextGeneration() {
        goToNextState(calculateNextStates());
    }

    private boolean[][] calculateNextStates() {
    	boolean[][] nextStates = new boolean[getNumberOfRows()][getNumberOfColumns()];
        for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
            	nextStates[rowIndex][columnIndex] = calculateNextState(rowIndex, columnIndex, cells[rowIndex][columnIndex]);
            }
        }
        return nextStates;
    	
    }

    private boolean calculateNextState(int rowIndex, int columnIndex, Cell cell) {
    	int aliveNeighbours = countAliveNeighbours(rowIndex, columnIndex);
    	if (cell.isDead() && aliveNeighbours == 3) {
    		return true;
    	}
    	if (cell.isAlive() && ( aliveNeighbours == 2 || aliveNeighbours == 3) ) {
    		return true;
    	}
    	return false;
    }

    private int countAliveNeighbours(int rowIndex, int columnIndex) {
    	List<Cell>list = getNeighbours(rowIndex, columnIndex);
    	int count = 0;
    	for (Cell Cell : list) {
    		if (Cell.isAlive()) {
    			count++;
    		}   		
    	}
        return count;
    }


    private List<Cell> getNeighbours(int rowIndex, int columnIndex) {
    	List<Cell> neighbours = new ArrayList<Cell>();
    	neighbours.add(getCell(rowIndex - 1, columnIndex - 1));//1 up left
    	neighbours.add(getCell(rowIndex - 1, columnIndex));//2 up 
    	neighbours.add(getCell(rowIndex - 1, columnIndex +1));//3 up right
    	neighbours.add(getCell(rowIndex, columnIndex + 1));//4 right
    	neighbours.add(getCell(rowIndex + 1, columnIndex + 1));//5 down right
    	neighbours.add(getCell(rowIndex + 1, columnIndex));//6 down 
    	neighbours.add(getCell(rowIndex + 1, columnIndex - 1));//7 down left
    	neighbours.add(getCell(rowIndex, columnIndex - 1));//8 left
	return neighbours;
    }

    private void goToNextState(boolean[][] nextState) {
    	
    	for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
            	if (nextState[rowIndex][columnIndex] == true) {
            		cells[rowIndex][columnIndex].setAlive();
            		
            			if ( neighboursColor(rowIndex, columnIndex) ) {
            				cells[rowIndex][columnIndex].setRed();
            			}
            			else 
            				cells[rowIndex][columnIndex].setBlue();
            		 
            	}
            	else
            		cells[rowIndex][columnIndex].setDead();
            		
            	
            	
    	
            }
    	
    	}
    }
    
    public boolean neighboursColor(int rowIndex, int columnIndex) {
    	List<Cell>color = getNeighbours(rowIndex, columnIndex);
    	int blue = 0;
    	int red = 0;
    	for (Cell Cell : color) {
    		if (Cell.isRed()) {
    			red++;
    			}   		
    		if (Cell.isBlue()){
    			blue++;
    			}   		
    	}
    	
        return red>blue;
    }

    /**
     * Sets all {@link Cell}s in this {@code Grid} as dead.
     */
    void clear() {
    	for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
            	cells[rowIndex][columnIndex].setDead();
            }
    	}
    }

    /**
     * Goes through each {@link Cell} in this {@code Grid} and randomly sets it as alive or dead.
     *
     * @param random {@link Random} instance used to decide if each {@link Cell} is alive or dead
     * @throws NullPointerException if {@code random} is {@code null}
     */
 
    void randomGeneration(Random random) {
    	for (int rowIndex = 0; rowIndex < getNumberOfRows(); rowIndex++) {
            for (int columnIndex = 0; columnIndex < getNumberOfColumns(); columnIndex++) {
            	
            	if (random.nextBoolean()) {
            	cells[rowIndex][columnIndex].setAlive();
            		if (random.nextBoolean()) {
            			cells[rowIndex][columnIndex].setBlue();
            			}
            		else 
            			cells[rowIndex][columnIndex].setRed();
            		}
            	else 
            		cells[rowIndex][columnIndex].setDead();
            }

    	}
    }
} 
